@@ -121,10 +121,12 @@ class OneBot11ForwardAdapter(BaseAdapter):
                     login = self.logins[self_id]
                     handler = events.get(event_type)
                     if not handler:
-                        event = Event(EventType.INTERNAL, datetime.now(), login, _type=event_type, _data=data)
+                        event = Event(EventType.INTERNAL, datetime.now(), login)
                     else:
                         event = await handler(login, self, data)
                     if event:
+                        event._type = event_type
+                        event._data = data
                         await self.server.post(event)
 
             asyncio.create_task(event_parse_task(data))
@@ -217,7 +219,7 @@ class OneBot11ForwardAdapter(BaseAdapter):
         async with self.session.get(path) as resp:
             return Response(await resp.read())
 
-    async def call_api(self, action: str, params: dict | None = None) -> dict | None:
+    async def call_api(self, action: str, params: dict | None = None) -> dict:
         if not self.connection:
             raise RuntimeError("connection is not established")
 
@@ -234,7 +236,7 @@ class OneBot11ForwardAdapter(BaseAdapter):
         if result["status"] != "ok":
             raise ActionFailed(f"{result['retcode']}: {result}", result)
 
-        return result.get("data")
+        return result.get("data", {})
 
     def __str__(self):
         return self.id
